diff --git a/src/main.cpp b/src/main.cpp
index a61fb67..f075504 100644
--- a/src/main.cpp
+++ b/src/main.cpp
@@ -7,6 +7,7 @@
 #include <array>
 #include <set>
 #include <memory>
+#include <optional>
 #include <stdexcept>
 
 class HelloTriangleApplication {
@@ -44,6 +45,7 @@ private:
     vk::raii::Context m_context;
     vk::raii::Instance m_instance{ nullptr };
     vk::raii::DebugUtilsMessengerEXT m_debugMessenger{ nullptr };
+    vk::raii::PhysicalDevice m_physicalDevice{ nullptr };
     /////////////////////////////////////////////////////////////////
 
     /////////////////////////////////////////////////////////////////
@@ -60,6 +62,7 @@ private:
     void initVulkan() {
         createInstance();
         setupDebugMessenger();
+        pickPhysicalDevice();
     }
 
     void mainLoop() {
@@ -171,6 +174,57 @@ private:
         m_debugMessenger = m_instance.createDebugUtilsMessengerEXT( createInfo );
     }
     /////////////////////////////////////////////////////////////////
+
+    /////////////////////////////////////////////////////////////////
+    /// physical device
+    struct QueueFamilyIndices {
+        std::optional<uint32_t> graphicsFamily;
+
+        bool isComplete() {
+            return graphicsFamily.has_value();
+        }
+    };
+    bool isDeviceSuitable(const vk::raii::PhysicalDevice& physicalDevice) {
+        QueueFamilyIndices indices = findQueueFamilies(physicalDevice);
+
+        return indices.isComplete();
+    }
+    void pickPhysicalDevice() {
+        // std::vector<vk::raii::PhysicalDevice>
+        auto physicalDevices = m_instance.enumeratePhysicalDevices();
+        if(physicalDevices.empty()){
+            throw std::runtime_error("failed to find GPUs with Vulkan support!");
+        }
+
+        for (const auto& it : physicalDevices) {
+            if (isDeviceSuitable(it)) {
+                m_physicalDevice = it;
+                break;
+            }
+        }
+        if(m_physicalDevice == nullptr){
+            throw std::runtime_error("failed to find a suitable GPU!");
+        }
+    }
+    QueueFamilyIndices findQueueFamilies(const vk::raii::PhysicalDevice& physicalDevice) {
+        QueueFamilyIndices indices;
+
+        // std::vector<vk::QueueFamilyProperties>
+        auto queueFamilies = physicalDevice.getQueueFamilyProperties();
+        for (int i = 0; const auto& queueFamily : queueFamilies) {
+            if (queueFamily.queueFlags & vk::QueueFlagBits::eGraphics) {
+                indices.graphicsFamily = i;
+            }
+            if (indices.isComplete()) {
+                break;
+            }
+
+            ++i;
+        }
+
+        return indices;
+    }
+    /////////////////////////////////////////////////////////////////
 };
 
 int main() {
